package main

import (
	"fmt"
	"github.com/gin-contrib/cors"
	"github.com/gin-gonic/gin"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"os"
	"strconv"
	"time"
)

type Wlzb struct {
	ID          int       `json:"id" gorm:"primaryKey"`
	PostID      string    `json:"post_id" gorm:"primaryKey"`
	CreateTime  time.Time `json:"create_time"`
	Author      string    `json:"author"`
	PostMessage string    `json:"post_message" gorm:"column:post_massage"`
}

func (Wlzb) TableName() string {
	return "wlzb"
}

func main() {
	r := gin.Default()

	// 自定义 CORS 中间件配置
	config := cors.DefaultConfig()
	config.AllowOrigins = []string{fmt.Sprintf("%s", os.Getenv("WEB_URL"))} // 允许的前端源
	config.AllowMethods = []string{"GET", "POST", "PUT", "PATCH", "DELETE", "HEAD", "OPTIONS"}
	config.AllowHeaders = []string{"Origin", "Content-Length", "Content-Type", "Authorization"}
	config.AllowCredentials = true

	r.Use(cors.New(config))

	// 数据库连接
	dsn := fmt.Sprintf("%s:%s@tcp(%s:%s)/%s?charset=utf8mb4&parseTime=True&loc=Local",
		os.Getenv("DB_USER"),
		os.Getenv("DB_PASSWORD"),
		os.Getenv("DB_HOST"),
		os.Getenv("DB_PORT"),
		os.Getenv("DB_NAME"))

	db, err := gorm.Open(mysql.Open(dsn), &gorm.Config{})
	if err != nil {
		panic("failed to connect database")
	}

	r.GET("/api/posts", func(c *gin.Context) {
		var posts []Wlzb
		pageStr := c.DefaultQuery("page", "1")
		limitStr := c.DefaultQuery("limit", "10")

		page, err := strconv.Atoi(pageStr)
		if err != nil {
			c.JSON(400, gin.H{"error": "Invalid page number"})
			return
		}

		limit, err := strconv.Atoi(limitStr)
		if err != nil {
			c.JSON(400, gin.H{"error": "Invalid limit number"})
			return
		}

		// 额外的检查
		if page < 1 {
			page = 1
		}

		if limit < 1 || limit > 100 {
			limit = 10
		}

		offset := (page - 1) * limit

		result := db.Order("id DESC").Limit(limit).Offset(offset).Find(&posts)
		if result.Error != nil {
			c.JSON(500, gin.H{"error": result.Error.Error()})
			return
		}

		c.JSON(200, posts)
	})

	// 新增的 API 路由，根据 ID 查询单个条目
	r.GET("/api/post/:id", func(c *gin.Context) {
		id, err := strconv.Atoi(c.Param("id"))
		if err != nil {
			c.JSON(400, gin.H{"error": "Invalid ID"})
			return
		}

		var post Wlzb
		result := db.First(&post, id)
		if result.Error != nil {
			if result.Error == gorm.ErrRecordNotFound {
				c.JSON(404, gin.H{"error": "Post not found"})
			} else {
				c.JSON(500, gin.H{"error": result.Error.Error()})
			}
			return
		}

		c.JSON(200, post)
	})

	r.Run(":8080")
}
